#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals

import os

from setuptools import setup, find_packages

from wiki import __version__

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...


def get_path(fname):
    return os.path.join(os.path.dirname(__file__), fname)


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


requirements = [
    "Django>=1.5",
    "Pillow",
    "django-nyt>=0.9.7.2,<1.0",
    "six",
]


# Requirements that depend on Django version: South and sorl-thumbnail
try:
    from django import VERSION as DJANGO_VERSION
except ImportError:
    # No Django so assuming that one will get installed, but we don't know which
    # one.
    # For Django 1.7, we don't need South at all, but installing it
    # doesn't harm anything (as long as it's not added to INSTALLED_APPS).
    # For Django <= 1.6, we need to ensure a recent South version,
    # so to be safe we include South here.
    requirements.append("South>=1.0.1")
    requirements.append("sorl-thumbnail>=12")
    # 0.6.1 broken: https://github.com/django-mptt/django-mptt/issues/316
    requirements.append("django-mptt>=0.8")
else:
    if DJANGO_VERSION < (1, 7):
        requirements.append("South>=1.0.1")
        requirements.append("django-mptt>=0.7.1,<0.8")
        requirements.append("django-sekizai<0.9")
    elif DJANGO_VERSION < (1, 8):
        # Fixes
        # AttributeError: 'URLPath' object has no attribute 'get_deferred_fields'
        requirements.append("django-mptt>=0.7.1,<0.8")
        requirements.append("django-sekizai>=0.9")
    else:
        # Latest django-mptt only works for Django 1.8+
        requirements.append("django-mptt>=0.8,<0.9")
        requirements.append("django-sekizai>=0.9")
    if DJANGO_VERSION < (1, 5):
        # For Django 1.4, use sorl-thumbnail<11.12.1:
        # https://github.com/mariocesar/sorl-thumbnail/issues/255
        requirements.append("sorl-thumbnail<11.12.1")
    else:
        requirements.append("sorl-thumbnail>=12,<13")

# Requirements that depend on Python version: Markdown
from sys import version_info as PYTHON_VERSION
if PYTHON_VERSION < (2, 7):
    # For Python 2.6, use Markdown<2.5.0, see
    # https://github.com/waylan/Python-Markdown/issues/349
    requirements.append("Markdown>2.2.0,<2.5.0")
else:
    requirements.append("Markdown>2.2.0,<2.7")

packages = find_packages()


setup(
    name="wiki",
    version=__version__,
    author="Benjamin Bach",
    author_email="benjamin@overtag.dk",
    url="http://www.django-wiki.org",
    description="A wiki system written for the Django framework.",
    license="GPLv3",
    keywords="django wiki markdown",
    packages=find_packages(exclude=["testproject", "testproject.*"]),
    long_description=read('README'),
    zip_safe=False,
    install_requires=requirements,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
    ],
    include_package_data=True,
    test_suite='runtests',
)



from __future__ import absolute_import
from __future__ import unicode_literals
from django import VERSION as DJANGO_VERSION
from django.conf.urls import patterns, url

from wiki.plugins.attachments import views

urlpatterns = [
    url(r'^$',
        views.AttachmentView.as_view(),
        name='attachments_index'),
    url(r'^search/$',
        views.AttachmentSearchView.as_view(),
        name='attachments_search'),
    url(r'^add/(?P<attachment_id>\d+)/$',
        views.AttachmentAddView.as_view(),
        name='attachments_add'),
    url(r'^replace/(?P<attachment_id>\d+)/$',
        views.AttachmentReplaceView.as_view(),
        name='attachments_replace'),
    url(r'^history/(?P<attachment_id>\d+)/$',
        views.AttachmentHistoryView.as_view(),
        name='attachments_history'),
    url(r'^download/(?P<attachment_id>\d+)/$',
        views.AttachmentDownloadView.as_view(),
        name='attachments_download'),
    url(r'^delete/(?P<attachment_id>\d+)/$',
        views.AttachmentDeleteView.as_view(),
        name='attachments_delete'),
    url(r'^download/(?P<attachment_id>\d+)/revision/(?P<revision_id>\d+)/$',
        views.AttachmentDownloadView.as_view(),
        name='attachments_download'),
    url(r'^change/(?P<attachment_id>\d+)/revision/(?P<revision_id>\d+)/$',
        views.AttachmentChangeRevisionView.as_view(),
        name='attachments_revision_change'),
]

if DJANGO_VERSION < (1, 8):
    urlpatterns = patterns('', *urlpatterns)



from __future__ import absolute_import
from __future__ import unicode_literals
# -*- coding: utf-8 -*-
from django.utils.translation import ugettext as _

from wiki.core.plugins import registry
from wiki.core.plugins.base import BasePlugin
from wiki.plugins.macros import settings

from wiki.plugins.macros.mdx.macro import MacroExtension
from wiki.plugins.macros.mdx.toc import WikiTocExtension
from wiki.plugins.macros.mdx.wikilinks import WikiLinkExtension


class MacroPlugin(BasePlugin):

    slug = settings.SLUG

    sidebar = {'headline': _('Macros'),
               'icon_class': 'fa-play',
               'template': 'wiki/plugins/macros/sidebar.html',
               'form_class': None,
               'get_form_kwargs': (lambda a: {})}

    markdown_extensions = [
        WikiLinkExtension(),
        MacroExtension(),
        WikiTocExtension()]

    def __init__(self):
        pass

registry.register(MacroPlugin)



from __future__ import absolute_import
from __future__ import unicode_literals
from django.conf import settings as django_settings

LOOKUP_LEVEL = getattr(django_settings, 'WIKI_LINKS_LOOKUP_LEVEL', 2)



from __future__ import print_function, unicode_literals

from io import BytesIO

from django.core.files.uploadedfile import InMemoryUploadedFile
from django.core.urlresolvers import reverse

from wiki.tests.base import ArticleWebTestBase


class AttachmentTests(ArticleWebTestBase):

    def setUp(self):
        super(AttachmentTests, self).setUp()
        self.article = self.root_article

    def _createTxtFilestream(self, strData, **kwargs):
        """
        Helper function to create filestream for upload.

        Parameters :
        strData : str, test string data

        Optional Arguments :
        filename : str, Defaults to 'test.txt'
        """
        filename = kwargs.get('filename', 'test.txt')
        data = strData.encode('utf-8')
        filedata = BytesIO(data)
        filestream = InMemoryUploadedFile(
            filedata,
            None,
            filename,
            'text',
            len(data),
            None
        )
        return filestream

    def test_upload(self):
        """
        Tests that simple file upload uploads correctly
        Uploading a file should preserve the original filename.
        Uploading should not modify file in any way.
        """
        url = reverse('wiki:attachments_index', kwargs={'path': ''})
        data = "This is a plain text file"
        filestream = self._createTxtFilestream(data)
        response = self.c.post(url,
                               {'description': 'My file',
                                'file': filestream,
                                'save': '1',
                                })
        self.assertRedirects(response, url)
        # Check the object was created.
        attachment = self.article.shared_plugins_set.all()[0].attachment
        self.assertEqual(attachment.original_filename, 'test.txt')
        self.assertEqual(attachment.current_revision.file.file.read(), data.encode('utf-8'))

    def test_replace(self):
        """
        Tests that previous revisions are not deleted
        Tests that only the most recent revision is deleted when
        "replace" is checked.
        """
        # Upload initial file
        url = reverse('wiki:attachments_index', kwargs={'path': ''})
        data = "This is a plain text file"
        filestream = self._createTxtFilestream(data)
        self.c.post(url, {'description': 'My file', 'file': filestream, 'save': '1', })
        attachment = self.article.shared_plugins_set.all()[0].attachment

        # uploading for the first time should mean that there is only one revision.
        self.assertEqual(attachment.attachmentrevision_set.count(), 1)

        # Change url to replacement page.
        url = reverse(
            'wiki:attachments_replace',
            kwargs={'attachment_id': attachment.id, 'article_id': self.article.id}
        )

        # Upload replacement without removing revisions
        replacement_data = data + ' And this is my edit'
        replacement_filestream = self._createTxtFilestream(replacement_data)
        self.c.post(
            url,
            {
                'description': 'Replacement upload',
                'file': replacement_filestream,
            }
        )
        attachment = self.article.shared_plugins_set.all()[0].attachment
        # Revision count should be two
        self.assertEqual(attachment.attachmentrevision_set.count(), 2)
        # Original filenames should not be modified
        self.assertEqual(attachment.original_filename, 'test.txt')
        # Latest revision should equal replacment_data
        self.assertEqual(attachment.current_revision.file.file.read(), replacement_data.encode('utf-8'))
        first_replacement = attachment.current_revision

        # Upload another replacement, this time removing most recent revision
        replacement_data2 = data + ' And this is a different edit'
        replacement_filestream2 = self._createTxtFilestream(replacement_data2)
        self.c.post(
            url,
            {
                'description': 'Replacement upload',
                'file': replacement_filestream2,
                'replace': 'on',
            }
        )
        attachment = self.article.shared_plugins_set.all()[0].attachment
        # Revision count should still be two
        self.assertEqual(attachment.attachmentrevision_set.count(), 2)
        # Latest revision should equal replacment_data2
        self.assertEqual(attachment.current_revision.file.file.read(), replacement_data2.encode('utf-8'))
        # The first replacement should no longer be in the filehistory
        self.assertNotIn(first_replacement, attachment.attachmentrevision_set.all())
